@@ -4,8 +4,9 @@
 #include "field.h"
 #include "random.h"
 #include "utils.h"
-//Naive implementation of F_p
-//may be preferable in some situations
+#include "fp.h"
+// naive implementation of F_p
+// basically wrappers around GMP mpz functions
 
 static void zp_init(element_ptr e)
 {
@@ -85,13 +86,26 @@ static void zp_sub(element_ptr n, element_ptr a, element_ptr b)
     }
 }
 
-static void zp_square(element_ptr n, element_ptr a)
+static void zp_square(element_ptr c, element_ptr a)
 {
     /*
-    mpz_mul(n->data, a->data, a->data);
-    mpz_mod(n->data, n->data, n->field->order);
+    mpz_mul(c->data, a->data, a->data);
+    mpz_mod(c->data, c->data, c->field->order);
+    */
+    mpz_powm_ui(c->data, a->data, 2, c->field->order);
+
+    /*
+    const mpz_ptr prime = c->field->order;
+    const size_t t = prime->_mp_size;
+    const mpz_ptr p = a->data;
+    const mpz_ptr r = c->data;
+    mp_limb_t tmp[2 * t];
+    mp_limb_t qp[t + 1];
+
+    mpn_mul_n(tmp, p->_mp_d, p->_mp_d, t);
+
+    mpn_tdiv_qr(qp, r->_mp_d, 0, tmp, 2 * t, prime->_mp_d, t);
     */
-    mpz_powm_ui(n->data, a->data, 2, n->field->order);
 }
 
 static void zp_double(element_ptr n, element_ptr a)
@@ -183,54 +197,6 @@ static int zp_is_sqr(element_ptr a)
     return mpz_legendre(a->data, a->field->order) == 1;
 }
 
-static void fp_tonelli(element_ptr x, element_ptr a)
-{
-    int s;
-    int i;
-    mpz_t e;
-    mpz_t t, t0;
-    element_t ginv, e0;
-    element_ptr nqr;
-
-    mpz_init(t);
-    mpz_init(e);
-    mpz_init(t0);
-    element_init(ginv, a->field);
-    element_init(e0, a->field);
-    nqr = field_get_nqr(a->field);
-
-    element_invert(ginv, nqr); 
-
-    //let q be the order of the field
-    //q - 1 = 2^s t, t odd
-    mpz_sub_ui(t, a->field->order, 1);
-    s = mpz_scan1(t, 0);
-    mpz_tdiv_q_2exp(t, t, s);
-    mpz_set_ui(e, 0);
-    for (i=2; i<=s; i++) {
-	mpz_sub_ui(t0, a->field->order, 1);
-	mpz_tdiv_q_2exp(t0, t0, i);
-	element_pow_mpz(e0, ginv, e);
-	element_mul(e0, e0, a);
-	element_pow_mpz(e0, e0, t0);
-	if (!element_is1(e0)) mpz_setbit(e, i-1);
-    }
-    element_pow_mpz(e0, ginv, e);
-    element_mul(e0, e0, a);
-    mpz_add_ui(t, t, 1);
-    mpz_tdiv_q_2exp(t, t, 1);
-    element_pow_mpz(e0, e0, t);
-    mpz_tdiv_q_2exp(e, e, 1);
-    element_pow_mpz(x, nqr, e);
-    /* TODO: this would be a good place to use element_pow2 ... -hs */
-    element_mul(x, x, e0);
-    mpz_clear(t);
-    mpz_clear(e);
-    mpz_clear(t0);
-    element_clear(ginv);
-    element_clear(e0);
-}
-
 static void zp_field_clear(field_t f)
 {
     UNUSED_VAR (f);
