@@ -285,11 +285,16 @@ element_ptr polymod_coeff(element_ptr e, int i)
     return coeff[i];
 }
 
+inline int polymod_field_degree(field_t f)
+{
+    polymod_field_data_ptr p = f->data;
+    return p->n;
+}
+
 static void polymod_random(element_ptr e)
 {
-    polymod_field_data_ptr p = e->field->data;
     element_t *coeff = e->data;
-    int i, n = p->n;
+    int i, n = polymod_field_degree(e->field);
 
     for (i=0; i<n; i++) {
 	element_random(coeff[i]);
@@ -299,9 +304,8 @@ static void polymod_random(element_ptr e)
 static void polymod_from_hash(element_ptr e, int len, void *data)
 {
     //TODO: improve this
-    polymod_field_data_ptr p = e->field->data;
     element_t *coeff = e->data;
-    int i, n = p->n;
+    int i, n = polymod_field_degree(e->field);
     for (i=0; i<n; i++) {
 	element_from_hash(coeff[i], len, data);
     }
@@ -310,9 +314,8 @@ static void polymod_from_hash(element_ptr e, int len, void *data)
 void polymod_const_mul(element_ptr res, element_ptr a, element_ptr e)
     //a lies in R, e in R[x]
 {
-    polymod_field_data_ptr p = e->field->data;
     element_t *coeff = e->data, *dst = res->data;
-    int i, n = p->n;
+    int i, n = polymod_field_degree(e->field);
 
     for (i=0; i<n; i++) {
 	element_mul(dst[i], coeff[i], a);
@@ -559,6 +562,56 @@ static void field_clear_poly(field_ptr f)
     free(p);
 }
 
+//2 bytes hold the number of terms
+//then the terms follow
+//bad for sparse polynomials
+
+static int poly_length_in_bytes(element_t p)
+{
+    int count = poly_coeff_count(p);
+    int result = 2;
+    int i;
+    for (i=0; i<count; i++) {
+	result += element_length_in_bytes(poly_coeff(p, i));
+    }
+    return result;
+}
+
+static int poly_to_bytes(unsigned char *buf, element_t p)
+{
+    int count = poly_coeff_count(p);
+    int result = 2;
+    int i;
+    buf[0] = (unsigned char) count;
+    buf[1] = (unsigned char) (count >> 8);
+    for (i=0; i<count; i++) {
+	result += element_to_bytes(&buf[result], poly_coeff(p, i));
+    }
+    return result;
+}
+
+static int poly_from_bytes(element_t p, unsigned char *buf)
+{
+    int result = 2;
+    int count = buf[0] + buf[1] * 256;
+    int i;
+    poly_alloc(p, count);
+    for (i=0; i<count; i++) {
+	result += element_from_bytes(poly_coeff(p, i), &buf[result]);
+    }
+    return result;
+}
+
+//Contrived? This returns to_mpz(constant term)
+static void poly_to_mpz(mpz_t z, element_ptr e)
+{
+    if (!poly_coeff_count(e)) {
+	mpz_set_ui(z, 0);
+    } else {
+	element_to_mpz(z, poly_coeff(e, 0));
+    }
+}
+
 void field_init_poly(field_ptr f, field_ptr base_field)
 {
     poly_field_data_ptr p;
@@ -573,6 +626,7 @@ void field_init_poly(field_ptr f, field_ptr base_field)
     f->clear = poly_clear;
     f->set_si = poly_set_si;
     f->set_mpz = poly_set_mpz;
+    f->to_mpz = poly_to_mpz;
     f->out_str = poly_out_str;
     f->set = poly_set;
     f->sign = poly_sign;
@@ -588,6 +642,11 @@ void field_init_poly(field_ptr f, field_ptr base_field)
     f->mul_mpz = poly_mul_mpz;
     f->mul_si = poly_mul_si;
     f->cmp = poly_cmp;
+
+    f->to_bytes = poly_to_bytes;
+    f->from_bytes = poly_from_bytes;
+    f->fixed_length_in_bytes = -1;
+    f->length_in_bytes = poly_length_in_bytes;
 }
 
 static void field_clear_polymod(field_ptr f)
@@ -1300,6 +1359,12 @@ static size_t polymod_out_str(FILE *stream, int base, element_ptr e)
     return result;
 }
 
+//Contrived? This returns to_mpz(constant term)
+static void polymod_to_mpz(mpz_t z, element_ptr e)
+{
+    element_to_mpz(z, polymod_coeff(e, 0));
+}
+
 static void compute_x_powers(field_ptr field, element_ptr poly)
 //compute x^n,...,x^{2n-2} mod poly
 {
@@ -1367,6 +1432,7 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     f->set0 = polymod_set0;
     f->set1 = polymod_set1;
     f->cmp = polymod_cmp;
+    f->to_mpz = polymod_to_mpz;
     switch(n) {
 	case 3:
 	    f->mul = polymod_mul_degree3;
@@ -1482,7 +1548,8 @@ int poly_is_irred_degfac(element_ptr f, darray_t factor)
     element_init(x, rxmod);
     element_init(g, f->field);
     n = factor->count;
-    element_set1(((element_t *) x->data)[1]);
+    //element_set1(((element_t *) x->data)[1]);
+    element_set1(polymod_coeff(x, 1));
 
     res = 0;
     for (i=0; i<n; i++) {
